/*
 * Public dpf interfaces. 
 */
#ifndef __DPF_H__
#define __DPF_H__

/* Errors */
enum { DPF_TOOMANYFILTERS = -2, DPF_TOOMANYELEMS = -3, DPF_BOGUSOP = -4, 
	DPF_BOGUSID = -5 , DPF_OVERLAP = -6, DPF_NILFILTER = -7};
	
/* Types we use.  Will be parameterized for the different machines. */
typedef unsigned int uint32;
typedef unsigned short uint16;
typedef unsigned char uint8;

#include <dpf-config.h>

/*
 * (*DANGER*): we rely on op, offset, nbits and mask being located in
 * the same place in struct eq and struct shift.
 */

/* size of the load is captured in the opcode. */
struct eq {
        uint8           op;     /* opcode. */
        uint16          offset; /* message offset */
        uint8           nbits;  /* length (in bits) */
        uint32          mask;   /* maximum width */
        uint32          val;    /* maximum size value */
};

/* size of the shift is captured in the opcode */
struct shift {
        uint8           op;     /* opcode */
        uint16          offset;
        uint8           nbits;  /* length (in bits) */
        uint32          mask;
        uint8           shift;
        uint8           align;  /* alignement given by shift */
};


struct ir {
        /**
         * The following fields are computed by dpf during insertion (we
         * don't trust clients).
         */

        /*
         * Maximum offset into message of code that we have to take
         * if this node succeeds.  Basically, for each or and shift
         * maxoffset holds the largest message offset of all nodes until
         * the next or, shift or end-of-filter.  This allows us to aggregate
         * message checks rather than having to do them on every atom.
         */
        uint16 maxoffset;

        /* Level of atom: is used as an index for various per-filter tables. */
        uint16 level;

        /* Alignment of atom at this point. */
        uint8   alignment;

        /* Whether a shift has occured upstream in the atomlist. */
        uint8 shiftp;

        union {
                struct eq eq;
                struct shift shift;
        } u;
};

/* container for code. */
struct dpf_ir {
        uint32  version;        /* ir may change from version to version. */
        uint16  irn;
        uint32  moffset;        /* the current offset added by dpf_shifti's. */
        struct ir ir[DPF_MAXELEM+1];            /* pointer to code */
};

/* Filter insertion/deletion. */
int dpf_insert(struct dpf_ir *filter);
int dpf_delete(unsigned pid);

/* dump a filter */
void dpf_printir(struct dpf_ir *ir);

/* initialize a filter structure. */
void dpf_init(struct dpf_ir *ir);

/* Called to classify a packet. */
extern int (*dpf_iptr)(uint8 *msg, unsigned nbytes);

/* 
 *  Filter creation routines.  nbits corresponds to 8, 16, 32 depending on
 * the operation.  msg[byte_offset:nbits] means to load nbits of the message
 * at byte_offset.
 */

/*
 * Compare message value to constant:
 * 	msg[byte_offset:nbits] == val
 */
void dpf_eq8(struct dpf_ir *ir, uint16 byte_offset, uint8 val);
void dpf_eq16(struct dpf_ir *ir, uint16 byte_offset, uint16 val);
void dpf_eq32(struct dpf_ir *ir, uint16 byte_offset, uint32 val);

/* 
 * Compare message value to constant:
 * 	msg[byte_offset:nbits] & mask == val
 */
void dpf_meq8(struct dpf_ir *ir, uint16 byte_offset, uint8 mask, uint8 val);
void dpf_meq16(struct dpf_ir *ir, uint16 byte_offset, uint16 mask, uint16 val);
void dpf_meq32(struct dpf_ir *ir, uint16 byte_offset, uint32 mask, uint32 val);

/* 
 * Shift the base message pointer: 
 * 	msg += msg[byte_offset:nbits] << shift; 
 */
void dpf_shift8(struct dpf_ir *ir, uint16 byte_offset, uint8 shift);
void dpf_shift16(struct dpf_ir *ir, uint16 byte_offset, uint8 shift);
void dpf_shift32(struct dpf_ir *ir, uint16 byte_offset, uint8 shift);

/* 
 * Shift the base message pointer: 
 *	msg += (msg[byte_offset:nbits] & mask) << shift; 
 */
void dpf_mshift8(struct dpf_ir *ir, uint16 offset, uint8 mask, uint8 shift);
void dpf_mshift16(struct dpf_ir *ir, uint16 offset, uint16 mask, uint8 shift);
void dpf_mshift32(struct dpf_ir *ir, uint16 offset, uint32 mask, uint8 shift);

/* Shift the base message pointer by a constant: msg += nbytes. */
void dpf_shifti(struct dpf_ir *ir, uint16 nbytes);


#endif /* __DPF_H__ */
